/**
 * @file  AE3Buffer.h
 * @brief Schnittstelle fuer die Klasse CAE3Buffer.
 */
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_)
#define AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// defines fuer Setup
#ifdef _WIN32
#include "../AESetup/Setup.h"

// DynData fuer Datensatzverwaltung im Kanal
#include "DynData.h"

// Geraetedatei einbinden
#include "../Common/DFFileDevice.h"

#else
#include "include/SetUp.h"

// Geraetedatei einbinden
#include "extern/DFFileDevice.h"

#endif


#pragma pack(1)

// Datensatzgroesse fuer Zutrittslisten definieren.
#define ENTRANCE_RECORD_SIZE_0    22
#define ENTRANCE_RECORD_SIZE_1    35
#define ENTRANCE_RECORD_SIZE_2    19
#define ENTRANCE_RECORD_SIZE_3    16

typedef struct structZeitZone
{

	UCHAR day;
	unsigned short start;
	unsigned short ende;
	UCHAR aktiv;

} STRUCTZeitZone;

typedef struct structZeitModell
{

	UCHAR zeitmodell;
	unsigned short zone;

} STRUCTZeitModell;

typedef struct structPersonalStamm
{

	UCHAR pNr[14];
	UCHAR zeitmodell;
	UCHAR prio;
	
} STRUCTPersonalStamm;

typedef struct structFeiertag
{

	UCHAR zeitmodell;
	ULONG datum;
	UCHAR prio;

} STRUCTFeiertag;

#pragma pack()

class CAE3Buffer
{
public:
	CAE3Buffer(unsigned char ucChannelNum);
	virtual ~CAE3Buffer();

	void ChangeOrderLong(unsigned char *Buf);
	unsigned long ChangeOrderULong(unsigned long val);
	void ULongToBinMSBFirst(unsigned char *Buf,unsigned long Val);
	void ChangeOrderInt(unsigned char *Buf);

protected:
	unsigned long StartDatensatzB, EndDatensatzB;
	int MaxListB;
	int MaxDatB;

	TDatensatzBeschreibung	DatensatzBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld			DatensatzFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD + 1];

	TDatensatzBeschreibung	ListenBeschreibung[MAX_DATENSATZ_BESCHREIBUNG];
	TDatensatzFeld			ListenFeld[MAX_DATENSATZ_BESCHREIBUNG][MAX_FELD + 1];

public:
	ULONG GetMDEEKFlashSize_0400();
	ULONG GetMDEEKFlashSize();
	ULONG GetEntranceEKFlashSize();
	ULONG GetFlashSetupSize();
	ULONG m_ulListSize, m_ulListStart;
	// Speicher fuer Listendaten
	unsigned char m_pucListMem[FLASH_SIZE_LIST_NEW];
	int MakeListe(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
	void ClearListBuffer();
	
	ULONG m_ulEntranceListSize[4];
	ULONG m_ulCompressedListSize[4];
	UCHAR m_ucEntranceRecordSize[4];
	// Speicher fuer Zutrittskontrolle - Listendaten
	unsigned char m_pucEntranceListMem[4][FLASH_SIZE_LIST + 1];
	unsigned char m_pucCompressedListMem[4][FLASH_SIZE_LIST + 1];
	int MakeEntranceList(int ListNum, int ListCnt, int ListSize, unsigned char *Buf);
	void ClearEntranceListBuffer(int iListNum);

	// Sortieren der Datensaetze ueber ASCII - Feld. Offs ist Offset in Datensatz auf Feldbeginn.
	void QuicksortEntranceRecord(unsigned char ucNum, unsigned char ucOffs, long lLeftRecordNum, long lRightRecordNum);
	void SwapEntranceRecord(unsigned char ucNum, long lLeftRecordNum, long lRightRecordNum);

	// fuer Setupdaten
	unsigned char m_pucEepMem[EEP_SIZE];
	unsigned char m_pucFlashMem[FLASH_SIZE_SU + 0x100];
	unsigned char m_pucConfigMem[CONFIG_SIZE_SU];
	unsigned char m_pucExtendedMem[EXTENDED_SIZE_SU + 0x100];

#ifdef _WIN32
	// Anfuegen von Daten nach alter Technik
	int AppendData(unsigned char *pucBuffer, int iLength);
#endif
	
	int GetTableCount(unsigned char ucList = FALSE);
	
	int GetTableRecord(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount, unsigned char *pucVariable = NULL, 
		unsigned char ucList = FALSE);

	int GetTableField(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength, unsigned char ucList = FALSE);

	int GetTableRecordEntrance(unsigned char ucNum,unsigned char *szName, 
		unsigned char *pucFieldCount);
	int GetTableFieldEntrance(unsigned char ucTableNum, unsigned char ucFieldNum, 
		unsigned char *szName, unsigned char *pucType, 
		unsigned char *pucLength);
	
	void DefaultIOModule(TAE3IOModule *pIOModule);

	// Datei fuer Geraetedaten (Firmware) verwalten
	CDFFileDevice m_fileDevice;

private:
	unsigned char m_ucChannelNum;

protected:
	void CpyFromKonfigBuf(unsigned char *Des,unsigned char *Src,unsigned long Add,int Cnt);
	
#ifdef _WIN32
	CDynData m_DataList;
#endif
};

#endif // !defined(AFX_AE3BUFFER_H__594E6CE6_B8A9_4D2C_A283_3AADA6D23606__INCLUDED_)

