/**
 * @file  DFLog.cpp
 * @brief Implementierung der Klasse CDFLog.
 */
//////////////////////////////////////////////////////////////////////

#include "StdAfx.h"
#include "DFLog.h"

#ifdef _WIN32
#include <direct.h>
#include <io.h>
#else
#include <sys/types.h>
#include <fcntl.h>
#include <unistd.h>
#include <dirent.h>
#endif

DFLog::DFLog()
{
#ifdef _DEBUG
	afxTraceEnabled = TRUE;
#endif
	// Initialisierungsdatei lesen
	IniRead();
	Check();
}

DFLog::~DFLog()
{
}

void DFLog::IniRead()
{
	char szTmp[MAX_PATH];
	char szFileName[MAX_PATH];
	sprintf(szFileName, INI_FILENAME);
	
	// pruefen und evnt. anlegen der Inidatei
	struct stat lstat;
	if (stat(szFileName, &lstat) != 0) {
		FILE *fp;
		if ((fp = fopen(szFileName, "w")) == (FILE*)0) {
			// Fehler bei erzeugung
			m_bEnable = false;
			m_bEraseFiles = false;
			return;
		}

		// Sektion "Log" anlegen
		fprintf(fp, "[Log]\n");
		fprintf(fp, "Enable=1\n");
#ifdef _WIN32
		// Aktuelles Verzeichnis verwenden
		memset(szTmp, 0, sizeof(szTmp));
		GetCurrentDirectory(MAX_PATH, szTmp);
		fprintf(fp, "Path=");
		for (int i=0; szTmp[i]; i++) {
			fprintf(fp, "%c", szTmp[i]);
			if (szTmp[i] == '\\') { // Maskierungszeichen einfuegen
				fprintf(fp, "\\");
			}
		}
		fprintf(fp, "\n");
#else
		fprintf(fp, "Path=./\n");
#endif
		fprintf(fp, "Folder=DFCom\n");
		fprintf(fp, "Erase=1\n");
		fprintf(fp, "Elapse=0\n");

		fclose(fp);
	}

	// Log aktiv?
	m_bEnable = (m_fileIni.GetUInt("Log", "Enable", 1, szFileName)) ? true : false;
	if (!m_bEnable) {

		return;
	}

	// Basispfad fuer Log
	memset(m_szBasePath, 0, sizeof(m_szBasePath));
	m_fileIni.GetString("Log", "Path", m_szBasePath, szFileName);
	if (m_szBasePath[0] == 0) {
		//fullpath(m_szBasePath, PATH_BASE, sizeof(m_szBasePath));
		getcwd(m_szBasePath, sizeof(m_szBasePath));
	} else {
#ifdef _WIN32
		if (m_szBasePath[strlen(m_szBasePath) - 1] != '\\') {
			strcat(m_szBasePath, "\\");
		}
#else
		if (m_szBasePath[strlen(m_szBasePath) - 1] != '/') {
			strcat(m_szBasePath, "/");
		}
#endif
	}
	// Unterordner fuer Logdateien
	memset(m_szSubFolder, 0, sizeof(m_szSubFolder));
	m_fileIni.GetString("Log", "Folder", m_szSubFolder, szFileName);
	if (m_szSubFolder[0] == 0) {
		strcpy(m_szSubFolder, SUB_FOLDER);
	}
	sprintf(szTmp, "%s%s", m_szBasePath, m_szSubFolder);
	// pruefen und evnt. anlegen des Ordners
	if (stat(szTmp, &lstat) != 0) {
#ifdef _WIN32
		if (mkdir(szTmp) != 0) {
#else
		if (mkdir(szTmp, 18) != 0) {
#endif
			// Fehler bei Erzeugung
			m_bEnable = false;
			m_bEraseFiles = false;
			return;
		}
	}
#ifdef _WIN32
	if (m_szSubFolder[strlen(m_szSubFolder) - 1] != '\\') {
		strcat(m_szSubFolder, "\\");
	}
#else
	if (m_szSubFolder[strlen(m_szSubFolder) - 1] != '/') {
		strcat(m_szSubFolder, "/");
	}
#endif
	// Dateiname der Logdatei
	time_t t = time(NULL);
	tm *ltm = localtime(&t);
	sprintf(m_szFileName, LOG_FILENAME, m_szBasePath, m_szSubFolder, 
		ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday);
	// Alte Dateien loeschen
	m_bEraseFiles = (m_fileIni.GetUInt("Log", "Erase", 1, szFileName)) ? true : false;
	// Ausfuehrungszeit angeben
	m_bElapse = (m_fileIni.GetUInt("Log", "Elapse", 1, szFileName)) ? true : false;
}

void DFLog::Check() 
{
	int i, j;
	char szFile[MAX_PATH];
	char szFileName[5][MAX_PATH];
	char szDeleteFileName[5][MAX_PATH];
	
	// Soll durchgefuehrt werden?
	if (!m_bEraseFiles) {
		
		return;
	}

	// Namen der Logdateien fuer die letzten fuenf Tage erstellen.
	for (i=0; i < 5; i++) {
		// Dateiname der Logdatei
		time_t t = time(NULL);
		t -= 3600 * 24 * i;
		tm *ltm = localtime(&t);
		sprintf(szFileName[i], LOG_FILENAME, m_szBasePath, m_szSubFolder, 
			ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday);
	}
	// Alle anderen Logdateien loeschen 


	j = 0;
#ifdef _WIN32

	// Suche nach Dateien
	sprintf(szFile, LOG_SEARCH_FILENAME, m_szBasePath, m_szSubFolder);
	long fp;
	struct finddata_t finddata;
	if ((fp = findfirst(szFile, &finddata)) != -1) {

		do {

			for (i=0; i < 5; i++) {
				// Stimmt mit einem der zu erhaltenden Dateien ueberein?
				if (strstr(szFileName[i], finddata.name) != 0) {
					break;
				}
			}
			if (i == 5) {
				// Sammeln der zu loeschenden Dateien,
				// da erst nach findclose geloescht werden kann!
				strcpy(szDeleteFileName[j++], finddata.name);
			} 

		} while ((j < 5) && (findnext(fp, &finddata) != -1));
		findclose(fp);

		if (j) {
			// Dateien loeschen
			for (i=0; i < j; i++) {
			
				sprintf(szFile, LOG_DELETE_FILENAME, m_szBasePath, m_szSubFolder, szDeleteFileName[i]);
				if (remove(szFile) == -1) {
					// Konnte nicht geloescht werden!
				}
			}
		}
	}
#else
	// Suche nach Dateien
	sprintf(szFile, "%s%s", m_szBasePath, m_szSubFolder);

	DIR *dir;
	if ((dir = opendir(szFile)) != NULL) {
		
		struct dirent *ent;
		while((j < 5) && ((ent = readdir(dir)) != NULL)) {

			for (i=0; i < 5; i++) {
				// Stimmt mit einem der zu erhaltenden Dateien ueberein?
				if (strstr(szFileName[i], ent->d_name) != 0) {
					break;
				}
			}
			if (i == 5) {
				// Sammeln der zu loeschenden Dateien,
				// da erst nach closedir geloescht werden kann!
				strcpy(szDeleteFileName[j++], ent->d_name);
			} 
		}
		closedir(dir);

		if (j) {
			// Dateien loeschen
			for (i=0; i < j; i++) {
			
				sprintf(szFile, LOG_DELETE_FILENAME, m_szBasePath, m_szSubFolder, szDeleteFileName[i]);
				if (remove(szFile) == -1) {
					// Konnte nicht geloescht werden!
				}
			}
		}
	}
#endif
	
}

void DFLog::Begin(const char *Format, ...)
{
	if (m_bEnable) {
		char szTime[9];
		char szFileName[MAX_PATH];
		char szLine[0x400];
		va_list ap;
		FILE *Logfp;

		// Uhrzeit des Logeintrages
		time_t t = time(NULL);
		m_clockBegin = clock();
		tm *ltm = localtime(&t);
		sprintf(szTime, "%02i:%02i:%02i", ltm->tm_hour, ltm->tm_min, ltm->tm_sec);
		// Dateiname der Logdatei
		sprintf(szFileName, LOG_FILENAME, m_szBasePath, m_szSubFolder, 
			ltm->tm_year % 100, ltm->tm_mon + 1, ltm->tm_mday);
		
		if (strcmp(szFileName, m_szFileName) != 0) {
			// Dateiname hat sich geaendert.
			strcpy(m_szFileName, szFileName);
			Check(); // Eventuell die aelteste Logdatei loeschen.
		}

		sprintf(szLine, "%s\t", szTime);
		va_start(ap, Format);
		vsprintf(&szLine[strlen(szLine)], Format, ap);
		va_end(ap);
		
		if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

			return;
		}

		fprintf(Logfp, szLine);

		fclose(Logfp);
	}
}

void DFLog::Append(const char *Format, ...)
{
	if (m_bEnable) {
		va_list ap;
		FILE *Logfp;

		if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

			return;
		}

		va_start(ap, Format);
		vfprintf(Logfp, Format, ap);
		va_end(ap);

		fclose(Logfp);
	}
}

void DFLog::End(const char *Format, ...)
{
	if (m_bEnable) {
		va_list ap;
		FILE *Logfp;

		if ((Logfp = fopen(m_szFileName, "a")) == (FILE *)0) {

			return;
		}

		va_start(ap, Format);
		vfprintf(Logfp, Format, ap);
		va_end(ap);

		// Ausfuehrungszeit angeben?
		if (m_bElapse) {
			// Ausfuehrungszeit berechnen
			double clockDiff;
			clockDiff = (double)(clock() - m_clockBegin) / CLOCKS_PER_SEC;

			fprintf(Logfp, "\t[%2.1f]\n", clockDiff);
		} else {

			fprintf(Logfp, "\n");
		}

		fclose(Logfp);
	}
}

