#include "StdAfx.h"
#include "DFFileDump.h"

#include <ctype.h>
#include <string.h>
#include <stdio.h>

CDFFileDump::CDFFileDump()
{
	Reset();
}

CDFFileDump::~CDFFileDump()
{
}

void CDFFileDump::Reset()
{
	// Fehlerobjekt zuruecksetzten
	m_errorState = ES_NO;
	
	// Basisadresse fuer Speichertyp
	m_baseAddress = 0;
	
	// Unterobjekte reseten
	m_flash.Reset();
}

// ########################################################
// HELPER

UINT32 CDFFileDump::HexToUINT32(INT8 *data)
{
	INT32 i;
	UINT32 val = 0;

	for (i=0; i < 8; i++) {

		val = (val << 4);

		if (data[i] >= '0' && data[i] <= '9') {

			val += data[i] - '0';

		} else if (data[i] >= 'a' && data[i] <='f') {

			val += data[i] - 'a' + 10;

		} else {

			val += data[i] - 'A' + 10;
		}
	}

	return val;
}

VOID CDFFileDump::UINT32ToHex(UINT32 val, INT8 *data)
{
	INT32 i;

	for (i=7; i >= 0; i--) {

		data[i] = val & 0xF;

		if (data[i] > 9) {
			
			data[i] += 'A' - 10; 
		} else {
		
			data[i] += '0';
		}

		val >>= 4;
	}	
}

UINT16 CDFFileDump::HexToUINT16(INT8 *data)
{
	INT32 i;
	UINT16 val = 0;

	for (i=0; i < 4; i++) {

		val = (val << 4);

		if (data[i] >= '0' && data[i] <= '9') {

			val += data[i] - '0';

		} else if (data[i] >= 'a' && data[i] <='f') {

			val += data[i] - 'a' + 10;

		} else {

			val += data[i] - 'A' + 10;
		}
	}

	return val;
}

VOID CDFFileDump::UINT16ToHex(UINT16 val, INT8 *data)
{
	INT32 i;

	for (i=3; i >= 0; i--) {

		data[i] = val & 0xF;

		if (data[i] > 9) {
			
			data[i] += 'A' - 10; 
		} else {
		
			data[i] += '0';
		}

		val >>= 4;
	}	
}

UINT8 CDFFileDump::HexToUINT8(INT8 *data)
{
	INT32 i;
	UINT8 val = 0;

	for (i=0; i < 2; i++) {

		val = (val << 4);

		if (data[i] >= '0' && data[i] <= '9') {

			val += data[i] - '0';

		} else if (data[i] >= 'a' && data[i] <='f') {

			val += data[i] - 'a' + 10;

		} else {

			val += data[i] - 'A' + 10;
		}
	}

	return val;
}

void CDFFileDump::UINT8ToHex(UINT8 val, INT8 *data)
{
	data[0] = (val >> 4) & 0xF;

	if (data[0] > 9) {
		
		data[0] += 'A' - 10; 
	} else {
		
		data[0] += '0';
	}

	data[1] = val & 0xF;

	if (data[1] > 9) {
		
		data[1] += 'A' - 10;
	} else {
		
		data[1] += '0';
	}
}

UINT32 CDFFileDump::GetAddress(INT8 *data)
{
	return HexToUINT32(&data[0]);
}

INT32 CDFFileDump::Load(INT8 *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	INT8 data[0x400];
//	UINT8 tmpData[0xFF];
	INT8 *pData;

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzten
	m_errorState = ES_NO;

	// Daten eintragen
//	INT8  count;
	UINT32 numLine = 0;
	UINT32 baseAddress = 0;
	UINT32 address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {

			strtok(data, " ");
			baseAddress = GetAddress(data);
			// erste Basisadresse halten
			if (!numLine) {
				m_baseAddress = baseAddress & 0xFF000000;
				m_flash.SetBaseAddress(baseAddress & 0x00FFFFFF);
			}

			// Vor erstes Datenbyte setzten
			pData = strtok(NULL, " \n");
			// Alle Datenbytes aus Zeile lesen
			for (address=baseAddress & 0x00FFFFFF; pData != NULL && pData[0] != ':'; pData = strtok(NULL, " \n"), address++) {

				if (address > (m_flash.GetBaseAddress() + DATAFLASH_SIZE)) {
					AfxMessageBox("Adressbereich berschritten.");
					break;
				}

				m_flash.SetMem(address, HexToUINT8(pData));
			}
			
			numLine++;
		}
	}
	fclose(fp);	

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		return FALSE;
	}

	return TRUE;
}

INT32 CDFFileDump::Save(INT8 *fileName) 
{
	// Schreiben der Hexdatei
	FILE *fp;
//	INT8 data[0x400];

	if ((fp = fopen(fileName, "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzten
	m_errorState = ES_NO;

	UINT32 numLine = 0;
	UINT8 lastUsebit = 0, sizeRecord = 0;
	UINT32 address = 0;
	UINT32 baseAddress = 0;

	UINT32 firstAddress = m_flash.GetMemFirstAddress();
	UINT32 lastAddress = m_flash.GetMemLastAddress();

	for (address=firstAddress; address <= lastAddress; address++) {

		// Adressangabe ausgeben
		if (!(address % 16) || !numLine) {

			if (sizeRecord) {
				UINT32 sub = address - sizeRecord;
				fprintf(fp, " : ");

				for (; sub < address; sub++) {
					
					fprintf(fp, "%c", isprint(m_flash.GetMem(sub)) ? m_flash.GetMem(sub) : 0x2E);
				}

				sizeRecord = 0;
			}

			fprintf(fp, "%s%08X ", (numLine) ? "\n" : "", m_baseAddress + address);
			numLine++;
		}

		// Wert ausgeben
		fprintf(fp, "%02X ", m_flash.GetMem(address));
		sizeRecord++;
	}
	if (sizeRecord) {
		UINT32 sub = address - sizeRecord;
		fprintf(fp, " : ");
		
		for (; sub < address; sub++) {
			
			fprintf(fp, "%c", isprint(m_flash.GetMem(sub)) ? m_flash.GetMem(sub) : 0x2E);
		}

		sizeRecord = 0;
	}


	fclose(fp);

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		return FALSE;
	}

	return TRUE;
}

